% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/consistentPaths.R
\docType{methods}
\name{checkPath}
\alias{checkPath}
\alias{checkPath,character,logical-method}
\alias{checkPath,character,missing-method}
\alias{checkPath,NULL,ANY-method}
\alias{checkPath,missing,ANY-method}
\title{Check filepath}
\usage{
checkPath(path, create)

\S4method{checkPath}{character,logical}(path, create)

\S4method{checkPath}{character,missing}(path)

\S4method{checkPath}{`NULL`,ANY}(path)

\S4method{checkPath}{missing,ANY}()
}
\arguments{
\item{path}{A character string corresponding to a filepath.}

\item{create}{A logical indicating whether the path should
be created if it doesn't exist. Default is \code{FALSE}.}
}
\value{
Character string denoting the cleaned up filepath.
}
\description{
Checks the specified filepath for formatting consistencies,
such as trailing slashes, etc.
}
\examples{
## normalize file paths
paths <- list("./aaa/zzz",
              "./aaa/zzz/",
              ".//aaa//zzz",
              ".//aaa//zzz/",
              ".\\\\aaa\\\\zzz",
              ".\\\\aaa\\\\zzz\\\\",
              file.path(".", "aaa", "zzz"))

checked <- normPath(paths)
length(unique(checked)) ## 1; all of the above are equivalent

## check to see if a path exists
tmpdir <- file.path(tempdir(), "example_checkPath")

dir.exists(tmpdir) ## FALSE
tryCatch(checkPath(tmpdir, create = FALSE), error = function(e) FALSE) ## FALSE

checkPath(tmpdir, create = TRUE)
dir.exists(tmpdir) ## TRUE

unlink(tmpdir, recursive = TRUE)
}
\seealso{
\code{\link{file.exists}}, \code{\link{dir.create}}.
}
