% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/postProcess.R
\name{projectInputs}
\alias{projectInputs}
\alias{projectInputs.Raster}
\alias{projectInputs.sf}
\alias{projectInputs.Spatial}
\title{Project \code{Raster*} or {Spatial*} or \code{sf} objects}
\usage{
projectInputs(x, targetCRS, ...)

\method{projectInputs}{Raster}(x, targetCRS = NULL,
  rasterToMatch = NULL, ...)

\method{projectInputs}{sf}(x, targetCRS, ...)

\method{projectInputs}{Spatial}(x, targetCRS, ...)
}
\arguments{
\item{x}{A \code{Raster*}, \code{Spatial*} or \code{sf} object}

\item{targetCRS}{The CRS of x at the end  of this function (i.e., the goal)}

\item{...}{Passed to \code{\link[raster]{projectRaster}}.}

\item{rasterToMatch}{Template \code{Raster*} object passed to the \code{to} argument of
\code{\link[raster]{projectRaster}}, thus will changing the
resolution and projection of \code{x}.
See details in \code{\link{postProcess}}.}
}
\value{
A file of the same type as starting, but with projection (and possibly other
characteristics, including resolution, origin, extent if changed.
}
\description{
A simple wrapper around the various different tools for these GIS types.
}
\examples{
# Add a study area to Crop and Mask to
# Create a "study area"
library(sp)
library(raster)
ow <- setwd(tempdir())

# make a SpatialPolygon
coords1 <- structure(c(-123.98, -117.1, -80.2, -100, -123.98, 60.9, 67.73, 65.58, 51.79, 60.9),
                    .Dim = c(5L, 2L))
Sr1 <- Polygon(coords1)
Srs1 <- Polygons(list(Sr1), "s1")
shpEcozone <- SpatialPolygons(list(Srs1), 1L)
crs(shpEcozone) <- "+init=epsg:4326 +proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0"

# make a "study area" that is subset of larger dataset
coords <- structure(c(-118.98, -116.1, -99.2, -106, -118.98, 59.9, 65.73, 63.58, 54.79, 59.9),
                    .Dim = c(5L, 2L))
Sr1 <- Polygon(coords)
Srs1 <- Polygons(list(Sr1), "s1")
StudyArea <- SpatialPolygons(list(Srs1), 1L)
crs(StudyArea) <- "+init=epsg:4326 +proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0"
#'
#'
##########
shpEcozonePostProcessed <- postProcess(shpEcozone, studyArea = StudyArea)
#'
# Try manually, individual pieces
shpEcozoneReprojected <- projectInputs(shpEcozone, StudyArea)
shpEcozoneCropped <- cropInputs(shpEcozone, StudyArea)
shpEcozoneClean <- fixErrors(shpEcozone)
shpEcozoneMasked <- maskInputs(shpEcozone, StudyArea)

setwd(ow)
}
